package org.acme.getting.started;

import org.jboss.logging.Logger;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@ApplicationScoped
public class MyThreadExecutor extends ScheduledThreadPoolExecutor {
    private static final Logger LOGGER = Logger.getLogger("ScheduledExecutorService");

    @Inject
    DataFileTask dataFileTask;

    public MyThreadExecutor() {
        super(1);
    }

    @Override
    protected void beforeExecute(Thread t, Runnable r) {
        super.beforeExecute(t, r);

        // Reschedule the dataFile task to run at a later stage
        super.schedule(dataFileTask, getDelaySeconds(), TimeUnit.SECONDS);
    }

    @Override
    protected void afterExecute(Runnable r, Throwable t) {
        super.afterExecute(r, t);

        LOGGER.info("The next data file creation task is scheduled to run just before: " + futureDate());
    }

    public void scheduleDataFileTask() {
        super.schedule(dataFileTask, 3, TimeUnit.SECONDS);
    }

    private long getDelaySeconds() {
        return ChronoUnit.SECONDS.between(LocalDateTime.now(), LocalDate.now().plusDays(1).atTime(1, 0));
    }

    private Date futureDate() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());

        calendar.add(Calendar.SECOND, (int) getDelaySeconds());

        return calendar.getTime();
    }
}
